@@ -21,7 +21,7 @@ use File::Temp 'tempdir';
 use ExtUtils::CBuilder;
 use Config;
 use DynaLoader;
-use SPVM::Util;
+use SPVM::Build;
 use File::Basename 'basename';
 
 
@@ -29,7 +29,7 @@ use Encode 'encode';
 
 use Carp 'confess';
 
-our $VERSION = '0.0261';
+our $VERSION = '0.0262';
 
 our $COMPILER;
 our @PACKAGE_INFOS;
@@ -60,7 +60,7 @@ sub import {
     }
   }
   
-  return undef;
+  return;
 }
 
 sub _get_dll_file {
@@ -69,31 +69,31 @@ sub _get_dll_file {
   # DLL file name
   my $dll_base_name = $package_name;
   $dll_base_name =~ s/^.*:://;
-  my $dll_file_tail = 'auto/' . $package_name . '/' . $dll_base_name;
-  $dll_file_tail =~ s/::/\//g;
-  my $dll_file;
+  my $shared_lib_file_tail = 'auto/' . $package_name . '.native' . '/' . $dll_base_name;
+  $shared_lib_file_tail =~ s/::/\//g;
+  my $shared_lib_file;
   for my $dl_shared_object (@DynaLoader::dl_shared_objects) {
     my $dl_shared_object_no_ext = $dl_shared_object;
     # remove .so, xs.dll .dll, etc
     while ($dl_shared_object_no_ext =~ s/\.[^\/\.]+$//) {
       1;
     }
-    if ($dl_shared_object_no_ext =~ /\Q$dll_file_tail\E$/) {
-      $dll_file = $dl_shared_object;
+    if ($dl_shared_object_no_ext =~ /\Q$shared_lib_file_tail\E$/) {
+      $shared_lib_file = $dl_shared_object;
       last;
     }
   }
   
-  return $dll_file;
+  return $shared_lib_file;
 }
 
 sub search_native_address {
-  my ($dll_file, $sub_abs_name) = @_;
+  my ($shared_lib_file, $sub_abs_name) = @_;
   
   my $native_address;
   
-  if ($dll_file) {
-    my $dll_libref = DynaLoader::dl_load_file($dll_file);
+  if ($shared_lib_file) {
+    my $dll_libref = DynaLoader::dl_load_file($shared_lib_file);
     if ($dll_libref) {
       my $sub_abs_name_c = $sub_abs_name;
       $sub_abs_name_c =~ s/:/_/g;
@@ -121,140 +121,44 @@ sub get_sub_native_address {
   }
   
   my $dll_package_name = $package_name;
-  my $dll_file = _get_dll_file($dll_package_name);
-  my $native_address = search_native_address($dll_file, $sub_abs_name);
+  my $shared_lib_file = _get_dll_file($dll_package_name);
+  my $native_address = search_native_address($shared_lib_file, $sub_abs_name);
   
-  # Search inline dlls
+  # Try runtime compile
   unless ($native_address) {
-    my $dll_file = compile_inline_native_sub($package_name);
-    if ($dll_file) {
-      $native_address = search_native_address($dll_file, $sub_abs_name);
+    
+    my $module_name = $package_name;
+    $module_name =~ s/^SPVM:://;
+    my $module_dir = get_use_package_path($module_name);
+    $module_dir =~ s/\.spvm$//;
+    
+    my $module_name_slash = $package_name;
+    $module_name_slash =~ s/::/\//g;
+    
+    $module_dir =~ s/$module_name_slash$//;
+    $module_dir =~ s/\/$//;
+    
+    my $shared_lib_file = SPVM::Build::build_shared_lib(
+      module_dir => $module_dir,
+      module_name => "SPVM::$module_name"
+    );
+    if ($shared_lib_file) {
+      $native_address = search_native_address($shared_lib_file, $sub_abs_name);
     }
   }
   
   return $native_address;
 }
 
-my $compiled = {};
-
-sub compile_inline_native_sub {
-  my $package_name = shift;
-  
-  $package_name =~ s/^SPVM:://;
-  
-  if ($compiled->{$package_name}) {
-    return $compiled->{$package_name};
-  }
-  
-  my $package_base_name = $package_name;
-  $package_base_name =~ s/^.+:://;
-  
-  my $shared_lib_dir = get_use_package_path($package_name);
-  $shared_lib_dir =~ s/\.spvm$//;
-  $shared_lib_dir .= '.native';
-  
-  my $src_file;
-  for my $file (glob "$shared_lib_dir/$package_base_name.*") {
-    next if $file =~ /\.config$/;
-    $src_file = $file;
-    last;
-  }
-  
-  my $config_file = "$shared_lib_dir/$package_base_name.config";
-  my $config;
-  if (-f $config_file) {
-  
-    $config = do $config_file
-      or confess "Can't parser $config_file: $!$@";
-  }
-  
-  my $api_header_include_dir = $INC{"SPVM.pm"};
-  $api_header_include_dir =~ s/\.pm$//;
-  
-  # Convert ExtUitls::MakeMaker config to ExtUtils::CBuilder config
-  my $cbuilder_new_config = {};
-  if ($config) {
-    # OPTIMIZE
-    if (defined $config->{OPTIMIZE}) {
-      $cbuilder_new_config->{optimize} = delete $config->{OPTIMIZE};
-    }
-    else {
-      # Default is -O3
-      $cbuilder_new_config->{optimize} = '-O3';
-    }
-    
-    # CC
-    if (defined $config->{CC}) {
-      $cbuilder_new_config->{cc} = delete $config->{CC};
-    }
-    
-    # CCFLAGS
-    if (defined $config->{CCFLAGS}) {
-      $cbuilder_new_config->{ccflags} = delete $config->{CCFLAGS};
-    }
-    
-    # LD
-    if (defined $config->{LD}) {
-      $cbuilder_new_config->{ld} = delete $config->{LD};
-    }
-    
-    # LDDLFLAGS
-    if (defined $config->{LDDLFLAGS}) {
-      $cbuilder_new_config->{lddlflags} = delete $config->{LDDLFLAGS};
-    }
-    
-    my @keys = keys %$config;
-    if (@keys) {
-      confess "$keys[0] is not supported option";
-    }
-  }
-  
-  my $package_name_under_score = $package_name;
-  $package_name_under_score =~ s/:/_/g;
-  
-  my $temp_dir = tempdir;
-  my $quiet = 1;
-  my $cbuilder = ExtUtils::CBuilder->new(quiet => $quiet, config => $config);
-  my $tmp_object_file = "$temp_dir/SPVM__${package_name_under_score}.o";
-  my $obj_file = $cbuilder->compile(
-    object_file => $tmp_object_file,
-    source => $src_file,
-    include_dirs => [$api_header_include_dir]
-  );
-  
-  
-  # This is required for Windows
-  my $native_sub_names = get_native_sub_names_from_package($package_name);
-  my $dl_func_list = [];
-  for my $native_sub_name (@$native_sub_names) {
-    my $dl_func = "SPVM__$native_sub_name";
-    $dl_func =~ s/:/_/g;
-    push @$dl_func_list, $dl_func;
-  }
-  
-  my $lib_file_name = "$temp_dir/SPVM__${package_name_under_score}.$Config{dlext}";
-  
-  my $lib_file = $cbuilder->link(
-    objects => $obj_file,
-    module_name => "SPVM::$package_name",
-    dl_func_list => $dl_func_list,
-    lib_file => $lib_file_name
-  );
-  
-  $compiled->{$package_name} = $lib_file;
-  
-  return $lib_file;
-}
-
 sub bind_native_subs {
-  my $native_sub_names = get_native_sub_names();
-  for my $native_sub_name (@$native_sub_names) {
-    my $native_sub_name_spvm = "SPVM::$native_sub_name";
-    my $native_address = get_sub_native_address($native_sub_name_spvm);
+  my $native_func_names = get_native_sub_names();
+  for my $native_func_name (@$native_func_names) {
+    my $native_func_name_spvm = "SPVM::$native_func_name";
+    my $native_address = get_sub_native_address($native_func_name_spvm);
     unless ($native_address) {
-      confess "Can't find native address($native_sub_name())";
+      confess "Can't find native address($native_func_name())";
     }
-    bind_native_sub($native_sub_name, $native_address);
+    bind_native_sub($native_func_name, $native_address);
   }
 }
 
@@ -265,13 +169,13 @@ CHECK {
   
   # Load standard library
   my @dll_file_bases = ('std', 'Math');
-  for my $dll_file_base (@dll_file_bases) {
-    my $dll_file_rel = "auto/SPVM/$dll_file_base.native/$dll_file_base.$Config{dlext}";
+  for my $shared_lib_file_base (@dll_file_bases) {
+    my $shared_lib_file_rel = "auto/SPVM/$shared_lib_file_base.native/$shared_lib_file_base.$Config{dlext}";
     for my $module_dir (@INC) {
-      my $dll_file = "$module_dir/$dll_file_rel";
-      if (-f $dll_file) {
-        DynaLoader::dl_load_file($dll_file);
-        push @DynaLoader::dl_shared_objects, $dll_file;
+      my $shared_lib_file = "$module_dir/$shared_lib_file_rel";
+      if (-f $shared_lib_file) {
+        DynaLoader::dl_load_file($shared_lib_file);
+        push @DynaLoader::dl_shared_objects, $shared_lib_file;
       }
     }
   }
